libname daruo 'C:\QE_MS_1528';run;


data crsp90yr;
set daruo.crsp90yr_raw;  *read in CRSP raw daily data;

if shrcd = 10 or shrcd = 11;  /*include only common shares*/

if exchcd=1 or exchcd=2 or exchcd=3; /*include only NYSE AMEX NASDAQ*/

if TRDSTAT='A';  /*Trading Status is ACTIVE*/

if ret>=-1;  /*ret values are not Missing*/

if dlret>=-1 then ret0=(ret+1)*(dlret+1)-1;  else ret0=ret;  /*adjust ret to include delisting returns*/

year=year(date);
month=month(date);
quarter=QTR(date);

keep permno date year quarter month ret0;
run;


data crsp90yr;
set crsp90yr;

if month<=11 then month_lead1 = month+1;
if month=12 then month_lead1 = 1;

if month<=11 then year_lead1 = year;
if month=12 then year_lead1 = year+1;

date_lead1 = mdy(month_lead1,01,year_lead1);
format date_lead1 mmddyy6.;
quarter_lead1=QTR(date_lead1);


keep permno ret0 date year quarter month year_lead1 quarter_lead1;

run;
*******************************************************************************;
*//find the number of valid observations each month each stock. then remove the stock-month with less than 15 observations//;

proc sort data=crsp90yr;  by permno year_lead1 quarter_lead1 date;run;

proc means data=crsp90yr noprint;
by permno year_lead1 quarter_lead1;
var date;
output out=num_days n=num_days;

data num_days; set num_days; label num_days='num_days'; drop _type_ _freq_;run;

proc sort data=num_days nodup; 
by year_lead1 quarter_lead1 permno;

proc means data=num_days noprint;
by year_lead1 quarter_lead1;
var num_days;
output out=max_days max=max_days;
data max_days; set max_days; label max_days='max_days'; drop _type_ _freq_;run;

data valid;
merge num_days max_days;
by year_lead1 quarter_lead1;
run;
*******************************************************************************;


*******************************************************************************;
proc sql;
     create table crsp90yr_valid
     as select x.*, s.*
     from crsp90yr as s
     left join  valid as  x
     on s.permno = x.permno and s.year_lead1 = x.year_lead1 and s.quarter_lead1 = x.quarter_lead1;
	 quit;
run;

data crsp90yr_valid;
set crsp90yr_valid;
if num_days >= (max_days - 3);  *require no more than 3 missing observations each quarter;
keep permno date year quarter year_lead1 quarter_lead1 ret0 num_days max_days;
run;
*******************************************************************************;

*//merge with Fama-French 4 daily factors//;
proc sql;
     create table crsp90yr_merged
     as select x.*, s.*
     from crsp90yr_valid as s
     left join  daruo.ff3_daily as  x
     on s.date = x.date;	 
quit;

**********************************************************************************;
*//save the merged dataset//;
proc sort data=crsp90yr_merged nodup;  by permno year_lead1 quarter_lead1 date; run;

data crsp90yr_merged;
set crsp90yr_merged;

ret=ret0-rf/100; *get stock excess returns;
Mkt_RF0 = Mkt_RF/100;
Mkt_RF_minus1=Mkt_RF_minus1/100;
Mkt_RF_minus2=Mkt_RF_minus2/100;
Mkt_RF_minus3=Mkt_RF_minus3/100;
Mkt_RF_minus4=Mkt_RF_minus4/100;

if Mkt_RF_minus4^=.;

keep permno date year_lead1 quarter_lead1 ret Mkt_RF0 Mkt_RF_minus1 Mkt_RF_minus2 Mkt_RF_minus3 Mkt_RF_minus4;
run;


*******************************************************************************************;
*******************************************************************************************;
*******************************************************************************************;


data crsp90yr_merged;
set crsp90yr_merged;
Mkt_RF1 = Mkt_RF_minus1;
Mkt_RF2 = (Mkt_RF_minus2 + Mkt_RF_minus3 + Mkt_RF_minus4);
drop Mkt_RF_minus1 Mkt_RF_minus2 Mkt_RF_minus3 Mkt_RF_minus4;
run;

proc sort data=crsp90yr_merged; by permno year_lead1 quarter_lead1 date; run;

proc reg noprint data=crsp90yr_merged outest=reg_results;
     model ret = Mkt_RF0 Mkt_RF1 Mkt_RF2/ADJRSQ;  *//reg model//;
     by permno year_lead1 quarter_lead1;
run;

data reg_results1;
set reg_results;
beta = Mkt_RF0 + Mkt_RF1 + Mkt_RF2; 
keep permno year_lead1 quarter_lead1 beta; 
run;

data daruo.cond_beta1;
set reg_results1;

month = quarter_lead1*3 - 1 ;
date = mdy(month,01,year_lead1);
format date mmddyy6.;

keep permno date beta;  
run;


