libname daruo 'C:\QE_MS_1528';run;


**********************;

data data0;
set daruo.jp_all_sw;

if ret_us^=.;

permno=firm;

format date mmddyy6.;

year=year(date);
month=month(date);
quarter=QTR(date);

ret0=ret_us;

keep permno date month quarter year ret0;
run;


*******************************************************************************;
*//find the number of valid observations each month each stock. then remove the stock-month with less than 15 observations//;

proc sort data=data0;  by permno year quarter month date;run;

proc means data=data0 noprint;
by permno year quarter;
var date;
output out=num_days n=num_days;

data num_days; set num_days; label num_days='num_days'; drop _type_ _freq_;run;

proc sort data=num_days nodup; 
by year quarter permno;

proc means data=num_days noprint;
by year quarter;
var num_days;
output out=max_days max=max_days;
data max_days; set max_days; label max_days='max_days'; drop _type_ _freq_;run;

data valid;
merge num_days max_days;
by year quarter;
run;
*******************************************************************************;


*******************************************************************************;
proc sql;
     create table data_valid
     as select x.*, s.*
     from data0 as s
     left join  valid as  x
     on s.permno = x.permno and s.year = x.year and s.quarter = x.quarter;
	 quit;
run;

data data_valid;
set data_valid;
if num_days >= (max_days - 3);  *require no more than 3 missing observations each quarter;
keep permno date year quarter month ret0 num_days max_days;
run;
*******************************************************************************;



*******************************************************************************;

*//merge with Fama-French 4 daily factors//;
proc sql;
     create table data_merged
     as select x.*, s.*
     from data_valid as s
     left join  daruo.jp_mkt_daily as  x
     on s.date = x.date;	 
quit;

**********************************************************************************;
*//save the merged dataset//;
proc sort data=data_merged nodup;  by permno year quarter month; run;

data data_merged;
set data_merged;

ret=ret0-rf; *get stock excess returns;
Mkt_RF0 = ret_mkt_ex;
Mkt_RF_minus1=lag1ret_mkt_ex;
Mkt_RF_minus2=lag2ret_mkt_ex;
Mkt_RF_minus3=lag3ret_mkt_ex;
Mkt_RF_minus4=lag4ret_mkt_ex;

if Mkt_RF_minus4^=.;

keep permno date year quarter month ret Mkt_RF0 Mkt_RF_minus1 Mkt_RF_minus2 Mkt_RF_minus3 Mkt_RF_minus4;
run;



*******************************************************************************************;
*******************************************************************************************;
*******************************************************************************************;

data data_merged;
set data_merged;
Mkt_RF1 = Mkt_RF_minus1;
Mkt_RF2 = (Mkt_RF_minus2 + Mkt_RF_minus3 + Mkt_RF_minus4);
drop Mkt_RF_minus1 Mkt_RF_minus2 Mkt_RF_minus3 Mkt_RF_minus4;
run;


proc sort data=data_merged; by permno year quarter date; run;

proc reg noprint data=data_merged outest=reg_results;
     model ret = Mkt_RF0 Mkt_RF1 Mkt_RF2/ADJRSQ;  *//reg model//;
     by permno year quarter;
run;

data reg_results1;
set reg_results;
beta = Mkt_RF0 + Mkt_RF1 + Mkt_RF2; 
keep permno year quarter beta; 
run;


data daruo.cond_beta0;
set reg_results1;

month = quarter*3;
date = mdy(month,01,year);
format date mmddyy6.;

keep permno date beta;  
run;
