
libname daruo 'C:\QE_MS_1528';run;


**********************;

data data;
set daruo.monthly_data_sw;
if ret_us^=.;

permno=firm_month;

date=mdy(month,01,year);
format date mmddyy6.;

keep permno date month year ret_us;
run;


data riskfree_rates;
set daruo.ff4_monthly;
keep date rf;
run;

data monthly_mkt_factor;
set daruo.monthly_mkt_factor;
keep date mkt;
run;

proc sql;  
     create table data01
     as select x.*, s.*
     from data as s
     left join riskfree_rates as  x
     on s.date = x.date;	 
quit;

proc sql;  
     create table data02
     as select x.*, s.*
     from data01 as s
     left join monthly_mkt_factor as  x
     on s.date = x.date;	 
quit;

data data0;
set data02;

ret1 = ret_us - rf/100;

mkt_rf = mkt;

keep permno date ret1 mkt_rf;
run;

proc sort data=data0;
by permno date;
run;




**********************************************************************************************;
**********************************************************************************************;

***find the first and last trading month for each stock**********************************************;
data begin (keep=permno bgndate) end (keep=permno endate);
set data0;
by permno;

if first.permno then do;
bgndate=date;
format bgndate mmddyy6.;
end;

if last.permno then do;
endate=date;
format endate mmddyy6.;
end;

data begin; set begin; if bgndate = . then delete;

data end; set end; if endate = . then delete;

*merge begin-dates with end-dates;
data length; merge begin end; by permno;

proc sql;  *merge stock IVOL with begin-dates and end-dates;
     create table data1
     as select x.*, s.*
     from data0 as s
     left join length as  x
     on s.permno = x.permno;	 
quit;

data data1; set data1; if permno^=' '; 
Proc sort data=data1 nodup; by permno date;

run;
**********************************************************************************************;
**********************************************************************************************;
**********************************************************************************************;


data data2;  *create three reference dates for the loop below;
set data1;  
date1_old=mdy((01),01,1980);
date2_old=mdy((01),01,1985);
format date1_old date2_old  mmddyy6.;
run;

************************************************************************************************************;
************************************************************************************************************;
*The loop starts at here;
************************************************************************************************************;
************************************************************************************************************;
ods graphics off;
ods exclude all;
ods noresults;



%MACRO DO_TIME;
%DO t = 200 %TO 299;   *1025;

**********************************************************************************************;
**********************************************************************************************;
proc printto log="nul:"; run;
**********************************************************************************************;

data data3;
set data2;
date1=intnx('month', date1_old, &t) + day(date1_old) -1;
date2=intnx('month', date2_old, &t) + day(date2_old) -1;
format date1 date2 mmddyy6.;
if date2 >= date;  
if date=date2 then ret1=.;
if bgndate > date1 or endate < date2  then delete; *at least 61 months as a time window;


data data3a;
set data3;
if date2 >= date >= date1;  

proc means data=data3a noprint;
by permno;
var date;
output out=count N=obvs;


data data4;
merge data3 count;
by permno;
if obvs>=61;

proc sort data=data4 nodup;
by permno date;
run;
************************************;
************************************;

proc datasets; delete nine_models; run;
proc datasets; delete min_AIC_model; run;


************************;


%DO i=1 %to 3;
    %DO j=1 %to 3;
	    ods noresults; 
        PROC AUTOREG DATA=data4 MAXITER=1000 OUTEST=status;
            BY permno;
            MODEL ret1= mkt_rf /garch=(p=&i, q=&j, type=exp);
            ODS OUTPUT FitSummary=Summary;
            OUTPUT OUT=eir CEV=vhat; 
        RUN;

        data aic0; set Summary; if Label2='AIC';
		data aic0; set aic0; by permno; if last.permno; AIC=nValue2; keep permno AIC; run;  


		data convergence; set status; if _STATUS_='0 Converged' then convergence=1; keep permno convergence; run;

		data IVOL;	set eir; by permno; if last.permno; if ret1=.; keep permno date vhat; run;

		data combination; merge IVOL aic0 convergence; by permno; if vhat^=.; run;

		PROC APPEND BASE=nine_models DATA=combination;


    %END;
%END;  


data nine_models; set nine_models; if convergence=1;

proc sort data=nine_models; by permno DESCENDING AIC; run;

data min_AIC_model; set nine_models; by permno; if last.permno; run;

************************************;
proc printto;  run;
************************************;

PROC APPEND BASE = daruo.Predicted_IVOL_part3  DATA=min_AIC_model;

************************************;


%END;
%MEND DO_TIME;

%do_time


run;
